@@ -23,7 +23,7 @@ module Text.TeXMath.Parser (parseFormula)
 where
 
 import Control.Monad
-import Data.Char (isAlphaNum, isDigit, isAscii)
+import Data.Char (isDigit, isAscii)
 import qualified Data.Map as M
 import Text.ParserCombinators.Parsec
 import qualified Text.ParserCombinators.Parsec.Token as P
@@ -82,6 +82,7 @@ formula = do
 
 expr :: TP Exp
 expr = do
+  optional (try $ symbol "\\displaystyle")
   a <- expr1
   limits <- limitsIndicator
   subSup limits a <|> superOrSubscripted limits a <|> return a
@@ -161,7 +162,7 @@ endLine = try $ do
   return '\n'
 
 arrayLine :: TP ArrayLine
-arrayLine = notFollowedBy (try $ char '\\' >> symbol "end" >> return '\n') >>
+arrayLine = notFollowedBy (try $ symbol "\\end" >> return '\n') >>
   sepBy1 (many (notFollowedBy endLine >> expr)) (symbol "&")
 
 arrayAlignments :: TP [Alignment]
@@ -175,13 +176,12 @@ arrayAlignments = try $ do
 
 environment :: TP Exp
 environment = try $ do
-  char '\\'
-  symbol "begin"
+  symbol "\\begin"
   name <- char '{' *> manyTill anyChar (char '}')
   spaces
   let name' = filter (/='*') name
   case M.lookup name' environments of
-        Just env -> env <* spaces <* char '\\' <* symbol "end"
+        Just env -> env <* spaces <* symbol "\\end"
                         <* braces (string name) <* spaces
         Nothing  -> mzero
 
@@ -246,27 +246,35 @@ variable = do
 
 isConvertible :: Exp -> Bool
 isConvertible (EMathOperator x) = x `elem` convertibleOps
-  where convertibleOps = ["lim","liminf","limsup","inf","sup"]
+  where convertibleOps = [ "lim","liminf","limsup","inf","sup"
+                         , "min","max","Pr","det","gcd"
+                         ]
 isConvertible (ESymbol Rel _) = True
 isConvertible (ESymbol Bin _) = True
-isConvertible (EUnder _ _)    = True
-isConvertible (EOver _ _)     = True
-isConvertible (EUnderover _ _ _) = True
 isConvertible (ESymbol Op x) = x `elem` convertibleSyms
   where convertibleSyms = ["\x2211","\x220F","\x22C2",
            "\x22C3","\x22C0","\x22C1","\x2A05","\x2A06",
            "\x2210","\x2A01","\x2A02","\x2A00","\x2A04"]
 isConvertible _ = False
 
+-- check if sub/superscripts should always be under and over the expression
+isUnderover :: Exp -> Bool
+isUnderover (EOver _ (ESymbol Accent "\xFE37")) = True   -- \overbrace
+isUnderover (EOver _ (ESymbol Accent "\x23B4")) = True   -- \overbracket
+isUnderover (EUnder _ (ESymbol Accent "\xFE38")) = True  -- \underbrace
+isUnderover (EUnder _ (ESymbol Accent "\x23B5")) = True  -- \underbracket
+isUnderover _ = False
+
 subSup :: Maybe Bool -> Exp -> TP Exp
 subSup limits a = try $ do
-  symbol "_"
-  b <- expr1
-  symbol "^"
-  c <- expr
+  let sub1 = symbol "_" >> expr1
+  let sup1 = symbol "^" >> expr1
+  (b,c) <- try (do {m <- sub1; n <- sup1; return (m,n)})
+       <|> (do {n <- sup1; m <- sub1; return (m,n)})
   return $ case limits of
             Just True  -> EUnderover a b c
             Nothing | isConvertible a -> EDownup a b c
+                    | isUnderover a -> EUnderover a b c
             _          -> ESubsup a b c
 
 superOrSubscripted :: Maybe Bool -> Exp -> TP Exp
@@ -278,17 +286,29 @@ superOrSubscripted limits a = try $ do
        '^' -> return $ case limits of
                         Just True  -> EOver a b
                         Nothing | isConvertible a -> EUp a b
+                                | isUnderover a -> EOver a b
                         _          -> ESuper a b
        '_' -> return $ case limits of
                         Just True  -> EUnder a b
                         Nothing | isConvertible a -> EDown a b
+                                | isUnderover a -> EUnder a b
                         _          -> ESub a b
        _   -> pzero
 
 escaped :: TP Exp
 escaped = lexeme $ try $
           char '\\' >>
-          liftM (ESymbol Ord . (:[])) (satisfy $ not . isAlphaNum)
+          liftM (ESymbol Ord . (:[])) (satisfy isEscapable)
+   where isEscapable '{' = True
+         isEscapable '}' = True
+         isEscapable '$' = True
+         isEscapable '%' = True
+         isEscapable '&' = True
+         isEscapable '_' = True
+         isEscapable '#' = True
+         isEscapable '^' = True  -- actually only if followed by {}
+         isEscapable ' ' = True
+         isEscapable _   = False
 
 unicode :: TP Exp
 unicode = lexeme $ liftM (ESymbol Ord . (:[])) $ satisfy (not . isAscii)
@@ -521,8 +541,8 @@ symbols = M.fromList [
            , ("\\nu", EIdentifier "\x03BD")
            , ("\\omega", EIdentifier "\x03C9")
            , ("\\Omega", ESymbol Op "\x03A9")
-           , ("\\phi", EIdentifier "\x03C6")
-           , ("\\varphi", EIdentifier "\x03D5")
+           , ("\\phi", EIdentifier "\x03D5")
+           , ("\\varphi", EIdentifier "\x03C6")
            , ("\\Phi", ESymbol Op "\x03A6")
            , ("\\pi", EIdentifier "\x03C0")
            , ("\\Pi", ESymbol Op "\x03A0")
